#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    DrPython is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#Plugins Menu

import os, os.path, shutil
import wx
import drScrolledMessageDialog
import drShortcutsFile
import drShortcuts
import drFileDialog
from drGetKeyDialog import drGetKeyDialog

class drPluginDialog(wx.Frame):

	def __init__(self, parent):
		wx.Frame.__init__(self, parent, -1, ("Configure Plugins"), wx.Point(50, 50), wx.Size(-1, -1), wx.DEFAULT_DIALOG_STYLE | wx.THICK_FRAME)
				
		wx.Yield()		
		
		if parent.PLATFORM_IS_WIN:
			self.SetSize((550, 425))
		
		#File Type
		self.wildcard = "DrPython Plugin (*.py)|*.py"
		self.iwildcard = "DrPython Plugin Index (*.idx)|*.idx|All files (*)|*"
		
		self.ID_CLOSE = 101
		self.ID_SAVE = 102
		
		self.ID_PROGRAM = 1001
		self.ID_POPUP = 1002
		
		self.ID_ADD = 1003
		self.ID_REMOVE = 1004
		self.ID_INDEX = 1005
		
		self.ID_UP = 1111
		self.ID_DOWN = 2222
		
		self.ID_INSTALL_PLUGIN = 4001
		self.ID_UNINSTALL_PLUGIN = 4002
		self.ID_INSTALL_PLUGIN_INDEX = 4003
		self.ID_UNINSTALL_PLUGIN_INDEX = 4004
		
		self.ID_PLUGINMENU = 4050
		self.ID_INDEXMENU = 4051
		self.ID_OPTIONSMENU = 4052
						
		self.ID_EDIT_PLUGIN = 4011
						
		self.theSizer = wx.FlexGridSizer(6, 4, 5, 10)
		self.menubuttonSizer = wx.BoxSizer(wx.VERTICAL)
		
		self.fileSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.homedirectory = parent.homedirectory
		
		plist = os.listdir(self.homedirectory + "/plugins")
				
		self.PluginList = []
		
		self.IndexList = ["<DEFAULT>"]
		
		self.indexname = ""
		self.indexplugins = []
		
		for p in plist:
			i = p.find(".py")			
			l = len(p)
			if i > -1 and (i + 3 == l):			
				self.PluginList.append(p[:i])
			else:
				i = p.find(".idx")
				if i > -1:
					self.IndexList.append(p)
		
		self.PluginList.sort()
		self.IndexList.sort()
		
		self.boxPlugins = wx.ListBox(self, self.ID_PROGRAM, wx.DefaultPosition, wx.Size(200, 200), self.PluginList)
		
		self.boxFile = wx.ListBox(self, self.ID_POPUP, wx.DefaultPosition, wx.Size(200, 200))
		
		self.btnAdd = wx.Button(self, self.ID_ADD, " ---> ")
		self.btnRemove = wx.Button(self, self.ID_REMOVE, " Remove ")
						
		self.btnUp = wx.Button(self, self.ID_UP, " Up ")
		self.btnDown = wx.Button(self, self.ID_DOWN, " Down ")
						
		self.cboIndex = wx.ComboBox(self, self.ID_INDEX, "<DEFAULT>", wx.DefaultPosition, (150, -1), self.IndexList, wx.CB_DROPDOWN)
		
		self.fileSizer.Add(wx.StaticText(self, -1, "Index:      "), 0, wx.SHAPED | wx.ALIGN_RIGHT)
		self.fileSizer.Add(self.cboIndex, 0, wx.SHAPED | wx.ALIGN_LEFT)
		
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(self.btnAdd, 0, wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(self.btnUp, 0, wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(self.btnDown, 0, wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.menubuttonSizer.Add(self.btnRemove, 0, wx.SHAPED)
		
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "Installed Plugins:"), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, "   "), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(self.fileSizer, 1, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(self.boxPlugins, 0, wx.SHAPED | wx.ALIGN_CENTER)
		self.theSizer.Add(self.menubuttonSizer, 0, wx.SHAPED | wx.ALIGN_CENTER)
		self.theSizer.Add(self.boxFile, 0,  wx.SHAPED | wx.ALIGN_CENTER)
		
		self.parent = parent
										
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		
		self.pluginmenu = wx.Menu()
		self.pluginmenu.Append(self.ID_INSTALL_PLUGIN, "&Install...")
		self.pluginmenu.Append(self.ID_UNINSTALL_PLUGIN, "&UnInstall...")
		self.pluginmenu.AppendSeparator()
		self.pluginmenu.Append(self.ID_EDIT_PLUGIN, "&Edit...")
				
		self.indexmenu = wx.Menu()
		self.indexmenu.Append(self.ID_INSTALL_PLUGIN_INDEX, "&Install...")
		self.indexmenu.Append(self.ID_UNINSTALL_PLUGIN_INDEX, "&UnInstall...")
		
		self.menuBar = wx.MenuBar()
		self.menuBar.Append(self.pluginmenu, "&Plugin")
		self.menuBar.Append(self.indexmenu, "&Index")
		
		self.SetMenuBar(self.menuBar)
		
		self.btnClose = wx.Button(self, self.ID_CLOSE, "&Close")
		self.btnSave = wx.Button(self, self.ID_SAVE, "&Save Index")
				
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(self.btnClose, 0,  wx.SHAPED | wx.ALIGN_CENTER)	
		self.theSizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_CENTER | wx.SHAPED)
		self.theSizer.Add(self.btnSave, 0,  wx.SHAPED | wx.ALIGN_CENTER)

		self.SetAutoLayout(True)
		self.SetSizerAndFit(self.theSizer)	
	
		self.Bind(wx.EVT_BUTTON, self.OnbtnUp, id=self.ID_UP)
		self.Bind(wx.EVT_BUTTON, self.OnbtnDown, id=self.ID_DOWN)
	
		self.Bind(wx.EVT_BUTTON, self.OnbtnAdd, id=self.ID_ADD)
		self.Bind(wx.EVT_BUTTON, self.OnbtnRemove, id=self.ID_REMOVE)
		self.Bind(wx.EVT_BUTTON, self.OnbtnClose, id=self.ID_CLOSE)
		self.Bind(wx.EVT_BUTTON, self.OnbtnSave, id=self.ID_SAVE)
				
		self.Bind(wx.EVT_MENU, self.OnUnInstallPlugin, id=self.ID_UNINSTALL_PLUGIN)
		self.Bind(wx.EVT_MENU, self.OnInstallPlugin, id=self.ID_INSTALL_PLUGIN)
		self.Bind(wx.EVT_MENU, self.OnInstallPluginIndex, id=self.ID_INSTALL_PLUGIN_INDEX)
		self.Bind(wx.EVT_MENU, self.OnUnInstallPluginIndex, id=self.ID_UNINSTALL_PLUGIN_INDEX)
				
		self.Bind(wx.EVT_MENU, self.OnEditPlugin, id=self.ID_EDIT_PLUGIN)
		
		self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnBoxClick, id=self.ID_PROGRAM)
		
		self.Bind(wx.EVT_COMBOBOX, self.OnOpen, id=self.ID_INDEX)
		self.Bind(wx.EVT_TEXT_ENTER, self.OncboNewIndex, id=self.ID_INDEX)
				
		self.parent.LoadDialogSizeAndPosition(self, 'plugindialog.sizeandposition.dat')
		
		self.OnOpen(None)

	def OnCloseW(self, event):
		self.parent.SaveDialogSizeAndPosition(self, 'plugindialog.sizeandposition.dat')
		if event is not None:
			event.Skip()	
			
	def EditPlugin(self, plugin):
		pluginfile = self.homedirectory + "/plugins/" + plugin + ".py"
		self.parent.DestroyRecentFileMenu()
		if len(self.parent.txtDocument.filename) > 0:
			self.parent.OpenFile(pluginfile, True)
		else:
			self.parent.OpenFile(pluginfile, False)
		self.parent.CreateRecentFileMenu()
		self.Close(True)
	
	def OnBoxClick(self, event):
		plugin = self.boxPlugins.GetStringSelection()
		d = wx.MessageDialog(self, 'Would you like to edit ' + plugin + '?', "Edit Plugin?", wx.YES_NO | wx.ICON_QUESTION)
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_YES):
			self.EditPlugin(plugin)
		self.Raise()
		self.SetFocus()			
	
	def OnbtnAdd(self, event):	
		tselection = self.boxPlugins.GetStringSelection()
		try:
			self.indexplugins.index(tselection)
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, 'Plugin "' + tselection + '" has already been added.', "Already Added")
			d.ShowModal()
			d.Destroy()
		except:			
			tsel = self.boxPlugins.GetSelection()
			if tsel == -1:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Nothing Selected to Add", "Mistake")
				d.ShowModal()
				d.Destroy()
				return
			sel = self.boxFile.GetSelection()
			if sel == -1:
				sel = 0	
			self.boxFile.Append(tselection)
			self.boxFile.SetSelection(sel)
			self.indexplugins.append(tselection)
	
	def OnbtnClose(self, event):
		self.Close(1)
	
	def OnbtnDown(self, event):
		sel = self.boxFile.GetSelection()
		if sel < self.boxFile.GetCount() - 1 and sel > -1:
			txt = self.boxFile.GetString(sel)
			self.boxFile.Delete(sel)
			self.boxFile.InsertItems([txt], sel+1)
			self.boxFile.SetSelection(sel+1)
	
	def OnbtnRemove(self, event):
		sel = self.boxFile.GetSelection()
		selection = self.boxFile.GetStringSelection()
		if sel == -1:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, "Nothing Selected to Remove", "Mistake")
			d.ShowModal()
			d.Destroy()
			return

		self.indexplugins.pop(sel)
		self.boxFile.Delete(sel)
		self.boxFile.SetSelection(sel-1)
	
	def OnbtnSave(self, event):
		f = file(self.indexname, 'w')
		for plugin in self.indexplugins:
			f.write(plugin + "\n")
		f.close()
		if self.parent.prefs.enablefeedback:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Succesfully wrote to:\n"  + self.indexname), "Saved Changes to Index File")
			d.ShowModal()
			d.Destroy()	
	
	def OnbtnUp(self, event):
		sel = self.boxFile.GetSelection()		
		if sel > 0:
			txt = self.boxFile.GetString(sel)
			self.boxFile.Delete(sel)
			self.boxFile.InsertItems([txt], sel-1)
			self.boxFile.SetSelection(sel-1)
	
	def OncboNewIndex(self, event):
		count = self.cboIndex.GetCount()
		self.cboIndex.Append(event.GetString())
		self.cboIndex.SetSelection(count)
		self.OnOpen(event)

	def OnEditPlugin(self, event):
		pluginsfile = self.homedirectory + "/plugins/plugins.dat"
		try:
			d = wx.SingleChoiceDialog(self.parent, "Select the Plugin to Edit:", "Edit Plugin", self.PluginList, wx.CHOICEDLG_STYLE)
			d.SetSize(wx.Size(250, 250))
			answer = d.ShowModal()
			d.Destroy()
			if (answer == wx.ID_OK):
				self.EditPlugin(d.GetStringSelection())
			else:
				self.Raise()
				self.SetFocus()
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Editing Plugin"), "Edit Plugin Error")
			d.ShowModal()
			d.Destroy()

	def OnEditShortcut(self, event):
		d = wx.SingleChoiceDialog(self.parent, "Select the Plugin:", "Edit Shortcuts", self.PluginList, wx.CHOICEDLG_STYLE)
		d.SetSize(wx.Size(250, 250))
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_OK):
			self.EditShortcut(d.GetStringSelection())		
		self.Raise()
		self.SetFocus()
	
	def OnHelp(self, event):
		d = wx.SingleChoiceDialog(self.parent, "Select the Plugin:", "Help", self.PluginList, wx.CHOICEDLG_STYLE)
		d.SetSize(wx.Size(250, 250))
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_OK):
			self.Help(d.GetStringSelection())
			return		
		self.Raise()
		self.SetFocus()
			
	def OnInstallPlugin(self, event):
		dlg = drFileDialog.FileDialog(self.parent, "Select Plugin to Install", self.wildcard)
		if (len(self.parent.prefs.pluginsdefaultdirectory) > 0):
			try:
				dlg.SetDirectory(self.parent.prefs.pluginsdefaultdirectory)			
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Setting Default Directory To: " + self.parent.prefs.pluginsdefaultdirectory), "DrPython Error")
				d.ShowModal()
				d.Destroy()
		if (dlg.ShowModal() == wx.ID_OK):
			if not os.path.exists(self.homedirectory + "/plugins"):
				os.mkdir(self.homedirectory + "/plugins")
			pluginfile = dlg.GetPath().replace("\\", "/")
			pluginrfile = self.homedirectory + "/plugins/" + os.path.split(pluginfile)[1]
			pluginpath, plugin = os.path.split(pluginfile)
			i = plugin.find(".py")
			if i == -1:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Plugins must be .py files."), "DrPython Error")
				d.ShowModal()
				d.Destroy()
			plugininstallfile = pluginfile + ".install"
			continueinstallation = True
			if os.path.exists(plugininstallfile):	
				f = open(plugininstallfile, 'r')
				scripttext = f.read()
				f.close()
				
				try:
					code = compile((scripttext + '\n'), plugininstallfile, 'exec')
				except:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error compiling install script."), "Error", wx.DefaultPosition, wx.Size(550,300))
					d.ShowModal()
					d.Destroy() 
					return
				
				try:
					cwd = os.getcwd()
					os.chdir(pluginpath)
					exec(code)
					continueinstallation = Install(self.parent)
					os.chdir(cwd)
				except:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error running install script."), "Error", wx.DefaultPosition, wx.Size(550,300))
					d.ShowModal()
					d.Destroy()
					return
			if not continueinstallation:
				return
			plugin = plugin[:i]
			pluginsfile = self.homedirectory + "/plugins/plugins.dat"
			try:
				copyf = True
				if os.path.exists(pluginrfile):
					d = wx.MessageDialog(self, 'Overwrite"' + pluginrfile + '"?', "DrPython", wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
					answer = d.ShowModal()
					d.Destroy()
					if (answer == wx.ID_NO):
						copyf = False
				if copyf:
					shutil.copyfile(pluginfile, pluginrfile)		
				try:
					f = file(pluginsfile, 'r')
					pluginstoload = f.read().rstrip().split('\n')
					f.close()				
				except:
					pluginstoload = []
				try:
					pluginstoload.index(plugin)
				except:
					f = file(pluginsfile, 'w')
					for p in pluginstoload:
						f.write(p + "\n")
					f.write(plugin)
					f.close()
				try:	
					self.parent.InitializePlugin(plugin, False)
				except:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error loading plugin: " + plugin), "Load Error")
					d.ShowModal()
					d.Destroy()		
				try:
					self.PluginList.index(plugin)
				except:
					self.PluginList.append(plugin)
				self.boxPlugins.Set(self.PluginList)
				self.indexname = self.homedirectory + "/plugins/plugins.dat"
				self.cboIndex.SetValue("<DEFAULT>")
				self.OnOpen(event)	
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error with: " + pluginfile), "Install Error")
				d.ShowModal()
				d.Destroy()
				return
		dlg.Destroy()
		self.Raise()
		self.SetFocus()	
		
	def OnInstallPluginIndex(self, event):
		dlg = drFileDialog.FileDialog(self.parent, "Select Plugin Index to Install", self.iwildcard)
		if (len(self.parent.prefs.pluginsdefaultdirectory) > 0):
			try:
				dlg.SetDirectory(self.parent.prefs.pluginsdefaultdirectory)			
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Setting Default Directory To: " + self.parent.prefs.pluginsdefaultdirectory), "DrPython Error")
				d.ShowModal()
				d.Destroy()
		if (dlg.ShowModal() == wx.ID_OK):
			if not os.path.exists(self.homedirectory + "/plugins"):
				os.mkdir(self.homedirectory + "/plugins")
			pindexfile = dlg.GetPath().replace("\\", "/")
			index = os.path.split(pindexfile)[1]
			pindexrfile = self.homedirectory + "/plugins/" + os.path.split(pindexfile)[1]
			try:
				copyf = True
				if os.path.exists(pindexrfile):
					d = wx.MessageDialog(self, 'Overwrite"' + pindexrfile + '"?', "DrPython", wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
					answer = d.ShowModal()
					d.Destroy()
					if (answer == wx.ID_NO):
						copyf = False
				if copyf:	
					shutil.copyfile(pindexfile, pindexrfile)				
				f = file(pindexrfile, 'r')
				pluginstoload = f.read().rstrip().split('\n')
				f.close()
				rootdir = os.path.split(pindexfile)[0]
				for plugin in pluginstoload:
					filename = rootdir + "/" + plugin + ".py"
					plugininstallfile = filename + ".install"
					continueinstallation = True
					if os.path.exists(plugininstallfile):	
						f = open(plugininstallfile, 'r')
						scripttext = f.read()
						f.close()
						
						try:
							code = compile((scripttext + '\n'), plugininstallfile, 'exec')
						except:
							d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error compiling install script."), "Error", wx.DefaultPosition, wx.Size(550,300))
							d.ShowModal()
							d.Destroy() 
							return						
						try:
							cwd = os.getcwd()
							os.chdir(rootdir)
							exec(code)
							continueinstallation = Install(self.parent)
							os.chdir(cwd)
						except:
							d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error running install script."), "Error", wx.DefaultPosition, wx.Size(550,300))
							d.ShowModal()
							d.Destroy()
							return
					if continueinstallation:
						target = self.homedirectory + "/plugins/" + plugin + ".py"
						shutil.copyfile(filename, target)
					try:
						self.PluginList.index(plugin)
					except:
						self.PluginList.append(plugin)
				self.boxPlugins.Set(self.PluginList)
				try:
					self.IndexList.index(index)
				except:
					self.IndexList.append(index)
				x = 0
				l = self.cboIndex.GetCount()
				while x < l:
					self.cboIndex.Delete(x)
					l = l - 1
				x = 0
				l = len(self.IndexList)
				while x < l:
					self.cboIndex.Append(self.IndexList[x])
					x = x + 1
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error with: " + pindexfile), "Install Index Error")
				d.ShowModal()
				d.Destroy()
				return
		dlg.Destroy()
		self.parent.pluginsmenu.reloadMenu()	
		self.Raise()
		self.SetFocus()
	
	def OnOpen(self, event):
		fname = self.cboIndex.GetValue()
		if fname == "<DEFAULT>":
			fname = "plugins.dat"
		self.indexname = self.homedirectory + "/plugins/" + fname
		if not os.path.exists(self.indexname):
			return
		try:
			f = file(self.indexname, 'r')
			self.indexplugins = f.read().rstrip().split('\n')
			f.close()
			
			x = 0
			l = len(self.indexplugins)
			while x < l:
				if len(self.indexplugins[x]) < 1:
					self.indexplugins.pop(x)
					x = x - 1
					l = l - 1
				x = x + 1
			
			self.boxFile.Set(self.indexplugins)
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Reading Index File: " + self.indexname), "Open Error")
			d.ShowModal()
			d.Destroy()			
	
	def OnPreferences(self, event):
		d = wx.SingleChoiceDialog(self.parent, "Select the Plugin:", "Plugin Preferences", self.PluginList, wx.CHOICEDLG_STYLE)
		d.SetSize(wx.Size(250, 250))
		answer = d.ShowModal()
		d.Destroy()
		if (answer == wx.ID_OK):
			self.EditPreferences(d.GetStringSelection())
		self.Raise()
		self.SetFocus()
			
	def OnUnInstallPlugin(self, event):
		pluginsfile = self.homedirectory + "/plugins/plugins.dat"
		try:
			d = wx.SingleChoiceDialog(self.parent, "Select the Plugin to Uninstall:", "UnInstall Plugin", self.PluginList, wx.CHOICEDLG_STYLE)
			d.SetSize(wx.Size(250, 250))
			answer = d.ShowModal()
			d.Destroy()
			if (answer == wx.ID_OK):
				f = file(pluginsfile, 'r')
				plugins = f.read().rstrip().split('\n')
				f.close()
				plugin = d.GetStringSelection()
				i = d.GetSelection()
				pluginfile = self.homedirectory + "/plugins/" + plugin + ".py"
				self.PluginList.pop(i)
				try:
					continueuninstall = True
					try:
						exec(compile("import " + plugin, plugin, 'exec'))
						exec(compile("continueuninstall = " + plugin + ".UnInstall(self.parent)", plugin, 'exec'))
					except:
						pass
					if continueuninstall:
						os.remove(pluginfile)
						if os.path.exists(pluginfile + "c"):
							os.remove(pluginfile + "c")
				except:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Removing Plugin File: " + pluginfile), "Uninstall Error")
					d.ShowModal()
					d.Destroy()
				try:
					i = plugins.index(plugin)
					plugins.pop(i)
					f = file(pluginsfile, 'w')
					for p in plugins:
						f.write(p + "\n")
					f.close()				
				except:
					pass
				self.boxPlugins.Set(self.PluginList)
				self.OnOpen(event)
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Uninstalling Plugin"), "Uninstall Error")
			d.ShowModal()
			d.Destroy()		
		self.Raise()
		self.SetFocus()

	def OnUnInstallPluginIndex(self, event):
		indexes = self.parent.pluginsmenu.indexes
		try:
			idxs = []
			for idx in indexes:
				idxs.append(os.path.split(idx)[1])
			d = wx.SingleChoiceDialog(self.parent, "Select the Plugin to Uninstall:", "UnInstall Plugin", idxs, wx.CHOICEDLG_STYLE)
			d.SetSize(wx.Size(250, 250))
			answer = d.ShowModal()
			d.Destroy()
			if (answer == wx.ID_OK):
				index = d.GetSelection()
				x = 0
				l = len(indexes)
				
				f = file(self.homedirectory + "/plugins/plugins.dat", 'r')
				pluginstokeep = f.read().rstrip().split('\n')
				f.close()
				
				while x < l:
					if not (x == index):
						f = file(indexes[x], 'r')
						pluginstokeep.extend(f.read().rstrip().split('\n'))
						f.close()
					x = x + 1
					
				f = file(indexes[index], 'r')
				pluginstoremove = f.read().rstrip().split('\n')
				f.close()
				
				try:
					os.remove(indexes[index])
					ili = self.IndexList.index(os.path.split(indexes[index])[1])
					rootdir = self.homedirectory + "/plugins/"
					for plugin in pluginstoremove:
						try:
							i = pluginstokeep.index(plugin)
						except:
							continueuninstall = True
							try:
								exec(compile("import " + plugin, plugin, 'exec'))
								exec(compile("continueuninstall = " + plugin + ".UnInstall(self.parent)", plugin, 'exec'))
							except:
								pass
							if continueuninstall:
								os.remove(rootdir + plugin + ".py")
								if os.path.exists(rootdir + plugin + ".pyc"):
									os.remove(rootdir + plugin + ".pyc")
							i = self.PluginList.index(plugin)
							self.PluginList.pop(i)
					self.boxPlugins.Set(self.PluginList)
					self.IndexList.pop(ili)
					indexes.pop(index)
					x = 0
					l = self.cboIndex.GetCount()
					while x < l:
						self.cboIndex.Delete(x)
						l = l - 1
					x = 0
					l = len(self.IndexList)
					while x < l:
						self.cboIndex.Append(self.IndexList[x])
						x = x + 1
				except:
					d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Removing index: " + indexes[index]), "Uninstall Error")
					d.ShowModal()
					d.Destroy()
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self.parent, ("Error Uninstalling Plugin Index"), "Uninstall Error")
			d.ShowModal()
			d.Destroy()
		self.parent.pluginsmenu.reloadMenu()
		self.Raise()
		self.SetFocus()